@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2014 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2016 Live Networks, Inc.  All rights reserved.
 // 'Group sockets'
 // Implementation
 
@@ -31,29 +31,27 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 
 OutputSocket::OutputSocket(UsageEnvironment& env)
   : Socket(env, 0 /* let kernel choose port */),
-    fSourcePort(0), fLastSentTTL(0) {
+    fSourcePort(0), fLastSentTTL(256/*hack: a deliberately invalid value*/) {
 }
 
 OutputSocket::OutputSocket(UsageEnvironment& env, Port port)
   : Socket(env, port),
-    fSourcePort(0), fLastSentTTL(0) {
+    fSourcePort(0), fLastSentTTL(256/*hack: a deliberately invalid value*/) {
 }
 
 OutputSocket::~OutputSocket() {
 }
 
-Boolean OutputSocket::write(netAddressBits address, Port port, u_int8_t ttl,
+Boolean OutputSocket::write(netAddressBits address, portNumBits portNum, u_int8_t ttl,
 			    unsigned char* buffer, unsigned bufferSize) {
-  if (ttl == fLastSentTTL) {
-    // Optimization: So we don't do a 'set TTL' system call again
-    ttl = 0;
-  } else {
-    fLastSentTTL = ttl;
-  }
   struct in_addr destAddr; destAddr.s_addr = address;
-  if (!writeSocket(env(), socketNum(), destAddr, port, ttl,
-		   buffer, bufferSize))
-    return False;
+  if ((unsigned)ttl == fLastSentTTL) {
+    // Optimization: Don't do a 'set TTL' system call again
+    if (!writeSocket(env(), socketNum(), destAddr, portNum, buffer, bufferSize)) return False;
+  } else {
+    if (!writeSocket(env(), socketNum(), destAddr, portNum, ttl, buffer, bufferSize)) return False;
+    fLastSentTTL = (unsigned)ttl;
+  }
 
   if (sourcePortNum() == 0) {
     // Now that we've sent a packet, we can find out what the
@@ -73,7 +71,7 @@ Boolean OutputSocket::write(netAddressBits address, Port port, u_int8_t ttl,
 // By default, we don't do reads:
 Boolean OutputSocket
 ::handleRead(unsigned char* /*buffer*/, unsigned /*bufferMaxSize*/,
-	     unsigned& /*bytesRead*/, struct sockaddr_in& /*fromAddress*/) {
+	     unsigned& /*bytesRead*/, struct sockaddr_in& /*fromAddressAndPort*/) {
   return True;
 }
 
@@ -81,9 +79,9 @@ Boolean OutputSocket
 ///////// destRecord //////////
 
 destRecord
-::destRecord(struct in_addr const& addr, Port const& port, u_int8_t ttl,
+::destRecord(struct in_addr const& addr, Port const& port, u_int8_t ttl, unsigned sessionId,
 	     destRecord* next)
-  : fNext(next), fGroupEId(addr, port.num(), ttl), fPort(port) {
+  : fNext(next), fGroupEId(addr, port.num(), ttl), fSessionId(sessionId) {
 }
 
 destRecord::~destRecord() {
@@ -103,8 +101,8 @@ Groupsock::Groupsock(UsageEnvironment& env, struct in_addr const& groupAddr,
 		     Port port, u_int8_t ttl)
   : OutputSocket(env, port),
     deleteIfNoMembers(False), isSlave(False),
-    fIncomingGroupEId(groupAddr, port.num(), ttl), fDests(NULL), fTTL(ttl) {
-  addDestination(groupAddr, port);
+    fDests(new destRecord(groupAddr, port, ttl, 0, NULL)),
+    fIncomingGroupEId(groupAddr, port.num(), ttl) {
 
   if (!socketJoinGroup(env, socketNum(), groupAddr.s_addr)) {
     if (DebugLevel >= 1) {
@@ -130,10 +128,8 @@ Groupsock::Groupsock(UsageEnvironment& env, struct in_addr const& groupAddr,
 		     Port port)
   : OutputSocket(env, port),
     deleteIfNoMembers(False), isSlave(False),
-    fIncomingGroupEId(groupAddr, sourceFilterAddr, port.num()),
-    fDests(NULL), fTTL(255) {
-  addDestination(groupAddr, port);
-
+    fDests(new destRecord(groupAddr, port, 255, 0, NULL)),
+    fIncomingGroupEId(groupAddr, sourceFilterAddr, port.num()) {
   // First try a SSM join.  If that fails, try a regular join:
   if (!socketJoinGroupSSM(env, socketNum(), groupAddr.s_addr,
 			  sourceFilterAddr.s_addr)) {
@@ -168,12 +164,26 @@ Groupsock::~Groupsock() {
   if (DebugLevel >= 2) env() << *this << ": deleting\n";
 }
 
+destRecord* Groupsock
+::createNewDestRecord(struct in_addr const& addr, Port const& port, u_int8_t ttl,
+		      unsigned sessionId, destRecord* next) {
+  // Default implementation:
+  return new destRecord(addr, port, ttl, sessionId, next);
+}
+
 void
 Groupsock::changeDestinationParameters(struct in_addr const& newDestAddr,
-				       Port newDestPort, int newDestTTL) {
-  if (fDests == NULL) return;
+				       Port newDestPort, int newDestTTL, unsigned sessionId) {
+  destRecord* dest;
+  for (dest = fDests; dest != NULL && dest->fSessionId != sessionId; dest = dest->fNext) {}
 
-  struct in_addr destAddr = fDests->fGroupEId.groupAddress();
+  if (dest == NULL) { // There's no existing 'destRecord' for this "sessionId"; add a new one:
+    fDests = createNewDestRecord(newDestAddr, newDestPort, newDestTTL, sessionId, fDests);
+    return;
+  }
+
+  // "dest" is an existing 'destRecord' for this "sessionId"; change its values to the new ones:
+  struct in_addr destAddr = dest->fGroupEId.groupAddress();
   if (newDestAddr.s_addr != 0) {
     if (newDestAddr.s_addr != destAddr.s_addr
 	&& IsMulticastAddress(newDestAddr.s_addr)) {
@@ -186,7 +196,7 @@ Groupsock::changeDestinationParameters(struct in_addr const& newDestAddr,
     destAddr.s_addr = newDestAddr.s_addr;
   }
 
-  portNumBits destPortNum = fDests->fGroupEId.portNum();
+  portNumBits destPortNum = dest->fGroupEId.portNum();
   if (newDestPort.num() != 0) {
     if (newDestPort.num() != destPortNum
 	&& IsMulticastAddress(destAddr.s_addr)) {
@@ -196,40 +206,42 @@ Groupsock::changeDestinationParameters(struct in_addr const& newDestAddr,
       socketJoinGroup(env(), socketNum(), destAddr.s_addr);
     }
     destPortNum = newDestPort.num();
-    fDests->fPort = newDestPort;
   }
 
   u_int8_t destTTL = ttl();
   if (newDestTTL != ~0) destTTL = (u_int8_t)newDestTTL;
 
-  fDests->fGroupEId = GroupEId(destAddr, destPortNum, destTTL);
+  dest->fGroupEId = GroupEId(destAddr, destPortNum, destTTL);
+
+  // Finally, remove any other 'destRecord's that might also have this "sessionId":
+  removeDestinationFrom(dest->fNext, sessionId);
 }
 
-void Groupsock::addDestination(struct in_addr const& addr, Port const& port) {
-  // Check whether this destination is already known:
-  for (destRecord* dests = fDests; dests != NULL; dests = dests->fNext) {
-    if (addr.s_addr == dests->fGroupEId.groupAddress().s_addr
-	&& port.num() == dests->fPort.num()) {
+unsigned Groupsock
+::lookupSessionIdFromDestination(struct sockaddr_in const& destAddrAndPort) const {
+  destRecord* dest = lookupDestRecordFromDestination(destAddrAndPort);
+  if (dest == NULL) return 0;
+
+  return dest->fSessionId;
+}
+
+void Groupsock::addDestination(struct in_addr const& addr, Port const& port, unsigned sessionId) {
+  // Default implementation:
+  // If there's no existing 'destRecord' with the same "addr", "port", and "sessionId", add a new one:
+  for (destRecord* dest = fDests; dest != NULL; dest = dest->fNext) {
+    if (sessionId == dest->fSessionId
+	&& addr.s_addr == dest->fGroupEId.groupAddress().s_addr
+	&& port.num() == dest->fGroupEId.portNum()) {
       return;
     }
   }
-
-  fDests = new destRecord(addr, port, ttl(), fDests);
+  
+  fDests = createNewDestRecord(addr, port, 255, sessionId, fDests);
 }
 
-void Groupsock::removeDestination(struct in_addr const& addr, Port const& port) {
-  for (destRecord** destsPtr = &fDests; *destsPtr != NULL;
-       destsPtr = &((*destsPtr)->fNext)) {
-    if (addr.s_addr == (*destsPtr)->fGroupEId.groupAddress().s_addr
-	&& port.num() == (*destsPtr)->fPort.num()) {
-      // Remove the record pointed to by *destsPtr :
-      destRecord* next = (*destsPtr)->fNext;
-      (*destsPtr)->fNext = NULL;
-      delete (*destsPtr);
-      *destsPtr = next;
-      return;
-    }
-  }
+void Groupsock::removeDestination(unsigned sessionId) {
+  // Default implementation:
+  removeDestinationFrom(fDests, sessionId);
 }
 
 void Groupsock::removeAllDestinations() {
@@ -247,14 +259,13 @@ void Groupsock::multicastSendOnly() {
 #endif
 }
 
-Boolean Groupsock::output(UsageEnvironment& env, u_int8_t ttlToSend,
-			  unsigned char* buffer, unsigned bufferSize,
+Boolean Groupsock::output(UsageEnvironment& env, unsigned char* buffer, unsigned bufferSize,
 			  DirectedNetInterface* interfaceNotToFwdBackTo) {
   do {
     // First, do the datagram send, to each destination:
     Boolean writeSuccess = True;
     for (destRecord* dests = fDests; dests != NULL; dests = dests->fNext) {
-      if (!write(dests->fGroupEId.groupAddress().s_addr, dests->fPort, ttlToSend,
+      if (!write(dests->fGroupEId.groupAddress().s_addr, dests->fGroupEId.portNum(), dests->fGroupEId.ttl(),
 		 buffer, bufferSize)) {
 	writeSuccess = False;
 	break;
@@ -269,14 +280,13 @@ Boolean Groupsock::output(UsageEnvironment& env, u_int8_t ttlToSend,
     if (!members().IsEmpty()) {
       numMembers =
 	outputToAllMembersExcept(interfaceNotToFwdBackTo,
-				 ttlToSend, buffer, bufferSize,
+				 ttl(), buffer, bufferSize,
 				 ourIPAddress(env));
       if (numMembers < 0) break;
     }
 
     if (DebugLevel >= 3) {
-      env << *this << ": wrote " << bufferSize << " bytes, ttl "
-	  << (unsigned)ttlToSend;
+      env << *this << ": wrote " << bufferSize << " bytes, ttl " << (unsigned)ttl();
       if (numMembers > 0) {
 	env << "; relayed to " << numMembers << " members";
       }
@@ -286,14 +296,16 @@ Boolean Groupsock::output(UsageEnvironment& env, u_int8_t ttlToSend,
   } while (0);
 
   if (DebugLevel >= 0) { // this is a fatal error
-    env.setResultMsg("Groupsock write failed: ", env.getResultMsg());
+    UsageEnvironment::MsgString msg = strDup(env.getResultMsg());
+    env.setResultMsg("Groupsock write failed: ", msg);
+    delete[] (char*)msg;
   }
   return False;
 }
 
 Boolean Groupsock::handleRead(unsigned char* buffer, unsigned bufferMaxSize,
 			      unsigned& bytesRead,
-			      struct sockaddr_in& fromAddress) {
+			      struct sockaddr_in& fromAddressAndPort) {
   // Read data from the socket, and relay it across any attached tunnels
   //##### later make this code more general - independent of tunnels
 
@@ -301,18 +313,19 @@ Boolean Groupsock::handleRead(unsigned char* buffer, unsigned bufferMaxSize,
 
   int maxBytesToRead = bufferMaxSize - TunnelEncapsulationTrailerMaxSize;
   int numBytes = readSocket(env(), socketNum(),
-			    buffer, maxBytesToRead, fromAddress);
+			    buffer, maxBytesToRead, fromAddressAndPort);
   if (numBytes < 0) {
     if (DebugLevel >= 0) { // this is a fatal error
-      env().setResultMsg("Groupsock read failed: ",
-			 env().getResultMsg());
+      UsageEnvironment::MsgString msg = strDup(env().getResultMsg());
+      env().setResultMsg("Groupsock read failed: ", msg);
+      delete[] (char*)msg;
     }
     return False;
   }
 
   // If we're a SSM group, make sure the source address matches:
   if (isSSM()
-      && fromAddress.sin_addr.s_addr != sourceFilterAddress().s_addr) {
+      && fromAddressAndPort.sin_addr.s_addr != sourceFilterAddress().s_addr) {
     return True;
   }
 
@@ -322,20 +335,20 @@ Boolean Groupsock::handleRead(unsigned char* buffer, unsigned bufferMaxSize,
   bytesRead = numBytes;
 
   int numMembers = 0;
-  if (!wasLoopedBackFromUs(env(), fromAddress)) {
+  if (!wasLoopedBackFromUs(env(), fromAddressAndPort)) {
     statsIncoming.countPacket(numBytes);
     statsGroupIncoming.countPacket(numBytes);
     numMembers =
       outputToAllMembersExcept(NULL, ttl(),
 			       buffer, bytesRead,
-			       fromAddress.sin_addr.s_addr);
+			       fromAddressAndPort.sin_addr.s_addr);
     if (numMembers > 0) {
       statsRelayedIncoming.countPacket(numBytes);
       statsGroupRelayedIncoming.countPacket(numBytes);
     }
   }
   if (DebugLevel >= 3) {
-    env() << *this << ": read " << bytesRead << " bytes from " << AddressString(fromAddress).val();
+    env() << *this << ": read " << bytesRead << " bytes from " << AddressString(fromAddressAndPort).val() << ", port " << ntohs(fromAddressAndPort.sin_port);
     if (numMembers > 0) {
       env() << "; relayed to " << numMembers << " members";
     }
@@ -346,10 +359,10 @@ Boolean Groupsock::handleRead(unsigned char* buffer, unsigned bufferMaxSize,
 }
 
 Boolean Groupsock::wasLoopedBackFromUs(UsageEnvironment& env,
-				       struct sockaddr_in& fromAddress) {
-  if (fromAddress.sin_addr.s_addr
-      == ourIPAddress(env)) {
-    if (fromAddress.sin_port == sourcePortNum()) {
+				       struct sockaddr_in& fromAddressAndPort) {
+  if (fromAddressAndPort.sin_addr.s_addr == ourIPAddress(env) ||
+      fromAddressAndPort.sin_addr.s_addr == 0x7F000001/*127.0.0.1*/) {
+    if (fromAddressAndPort.sin_port == sourcePortNum()) {
 #ifdef DEBUG_LOOPBACK_CHECKING
       if (DebugLevel >= 3) {
 	env() << *this << ": got looped-back packet\n";
@@ -362,6 +375,32 @@ Boolean Groupsock::wasLoopedBackFromUs(UsageEnvironment& env,
   return False;
 }
 
+destRecord* Groupsock
+::lookupDestRecordFromDestination(struct sockaddr_in const& destAddrAndPort) const {
+  for (destRecord* dest = fDests; dest != NULL; dest = dest->fNext) {
+    if (destAddrAndPort.sin_addr.s_addr == dest->fGroupEId.groupAddress().s_addr
+	&& destAddrAndPort.sin_port == dest->fGroupEId.portNum()) {
+      return dest;
+    }
+  }
+  return NULL;
+}
+
+void Groupsock::removeDestinationFrom(destRecord*& dests, unsigned sessionId) {
+  destRecord** destsPtr = &dests;
+  while (*destsPtr != NULL) {
+    if (sessionId == (*destsPtr)->fSessionId) {
+      // Remove the record pointed to by *destsPtr :
+      destRecord* next = (*destsPtr)->fNext;
+      (*destsPtr)->fNext = NULL;
+      delete (*destsPtr);
+      *destsPtr = next;
+    } else {
+      destsPtr = &((*destsPtr)->fNext);
+    }
+  }
+}
+
 int Groupsock::outputToAllMembersExcept(DirectedNetInterface* exceptInterface,
 					u_int8_t ttlToFwd,
 					unsigned char* data, unsigned size,
@@ -420,7 +459,8 @@ int Groupsock::outputToAllMembersExcept(DirectedNetInterface* exceptInterface,
 
       if (fDests != NULL) {
 	trailer->address() = fDests->fGroupEId.groupAddress().s_addr;
-	trailer->port() = fDests->fPort; // structure copy, outputs in network order
+	Port destPort(ntohs(fDests->fGroupEId.portNum()));
+	trailer->port() = destPort; // structure copy
       }
       trailer->ttl() = ttlToFwd;
       trailer->command() = tunnelCmd;
@@ -514,9 +554,7 @@ static Boolean setGroupsockBySocket(UsageEnvironment& env, int sock,
       = (sockets->Lookup((char*)(long)sock) != 0);
     if (alreadyExists) {
       char buf[100];
-      sprintf(buf,
-	      "Attempting to replace an existing socket (%d",
-	      sock);
+      sprintf(buf, "Attempting to replace an existing socket (%d)", sock);
       env.setResultMsg(buf);
       break;
     }
