@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2014 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2016 Live Networks, Inc.  All rights reserved.
 // A simple UDP sink (i.e., without RTP or other headers added); one frame per packet
 // Implementation
 
@@ -72,7 +72,7 @@ void BasicUDPSink::afterGettingFrame1(unsigned frameSize, unsigned numTruncatedB
   }
 
   // Send the packet:
-  fGS->output(envir(), fGS->ttl(), fOutputBuffer, frameSize);
+  fGS->output(envir(), fOutputBuffer, frameSize);
 
   // Figure out the time at which the next packet should be sent, based
   // on the duration of the payload that we just read:
