@@ -14,13 +14,12 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2014 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2016 Live Networks, Inc.  All rights reserved.
 // H.264 Video File sinks
 // Implementation
 
 #include "H264VideoFileSink.hh"
 #include "OutputFile.hh"
-#include "H264VideoRTPSource.hh"
 
 ////////// H264VideoFileSink //////////
 
@@ -28,8 +27,8 @@ H264VideoFileSink
 ::H264VideoFileSink(UsageEnvironment& env, FILE* fid,
 		    char const* sPropParameterSetsStr,
 		    unsigned bufferSize, char const* perFrameFileNamePrefix)
-  : FileSink(env, fid, bufferSize, perFrameFileNamePrefix),
-    fSPropParameterSetsStr(sPropParameterSetsStr), fHaveWrittenFirstFrame(False) {
+  : H264or5VideoFileSink(env, fid, bufferSize, perFrameFileNamePrefix,
+			 sPropParameterSetsStr, NULL, NULL) {
 }
 
 H264VideoFileSink::~H264VideoFileSink() {
@@ -58,25 +57,3 @@ H264VideoFileSink::createNew(UsageEnvironment& env, char const* fileName,
 
   return NULL;
 }
-
-void H264VideoFileSink::afterGettingFrame(unsigned frameSize, unsigned numTruncatedBytes, struct timeval presentationTime) {
-  unsigned char const start_code[4] = {0x00, 0x00, 0x00, 0x01};
-
-  if (!fHaveWrittenFirstFrame) {
-    // If we have PPS/SPS NAL units encoded in a "sprop parameter string", prepend these to the file:
-    unsigned numSPropRecords;
-    SPropRecord* sPropRecords = parseSPropParameterSets(fSPropParameterSetsStr, numSPropRecords);
-    for (unsigned i = 0; i < numSPropRecords; ++i) {
-      addData(start_code, 4, presentationTime);
-      addData(sPropRecords[i].sPropBytes, sPropRecords[i].sPropLength, presentationTime);
-    }
-    delete[] sPropRecords;
-    fHaveWrittenFirstFrame = True; // for next time
-  }
-
-  // Write the input data to the file, with the start code in front:
-  addData(start_code, 4, presentationTime);
-
-  // Call the parent class to complete the normal file write with the input data:
-  FileSink::afterGettingFrame(frameSize, numTruncatedBytes, presentationTime);
-}
