@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2014 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2016 Live Networks, Inc.  All rights reserved.
 // MP3 File Sources
 // Implementation
 
@@ -99,7 +99,6 @@ void MP3FileSource::seekWithinFile(double seekNPT, double streamDuration) {
       fNumBytesToStream = endByteNumber - seekByteNumber;
       fLimitNumBytesToStream = True;
     }
-  } else {
   }
 }
 
@@ -111,7 +110,7 @@ void MP3FileSource::getAttributes() const {
 
 void MP3FileSource::doGetNextFrame() {
   if (!doGetNextFrame1()) {
-    handleClosure(this);
+    handleClosure();
     return;
   }
 
