@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2014 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2016 Live Networks, Inc.  All rights reserved.
 // A filter that produces a sequence of I-frame indices from a MPEG-2 Transport Stream
 // Implementation
 
@@ -202,7 +202,13 @@ void MPEG2IFrameIndexFromTransportStream
   // Figure out how much of this Transport Packet contains PES data:
   u_int8_t adaptation_field_control = (fInputBuffer[3]&0x30)>>4;
   u_int8_t totalHeaderSize
-    = adaptation_field_control == 1 ? 4 : 5 + fInputBuffer[4];
+    = adaptation_field_control <= 1 ? 4 : 5 + fInputBuffer[4];
+  if ((adaptation_field_control == 2 && totalHeaderSize != TRANSPORT_PACKET_SIZE) ||
+      (adaptation_field_control == 3 && totalHeaderSize >= TRANSPORT_PACKET_SIZE)) {
+    envir() << "Bad \"adaptation_field_length\": " << fInputBuffer[4] << "\n";
+    doGetNextFrame();
+    return;
+  }
 
   // Check for a PCR:
   if (totalHeaderSize > 5 && (fInputBuffer[5]&0x10) != 0) {
@@ -241,7 +247,7 @@ void MPEG2IFrameIndexFromTransportStream
   // or packets with no data, or packets that duplicate the previous packet:
   u_int8_t continuity_counter = fInputBuffer[3]&0x0F;
   if ((PID != fVideo_PID) ||
-      !(adaptation_field_control == 1  || adaptation_field_control == 3) ||
+      !(adaptation_field_control == 1 || adaptation_field_control == 3) ||
       continuity_counter == fLastContinuityCounter) {
     doGetNextFrame();
     return;
@@ -250,8 +256,9 @@ void MPEG2IFrameIndexFromTransportStream
 
   // Also, if this is the start of a PES packet, then skip over the PES header:
   Boolean payload_unit_start_indicator = (fInputBuffer[1]&0x40) != 0;
-  if (payload_unit_start_indicator) {
-    // Note: The following works only for MPEG-2 data #####
+  if (payload_unit_start_indicator && totalHeaderSize < TRANSPORT_PACKET_SIZE - 8 
+      && fInputBuffer[totalHeaderSize] == 0x00 && fInputBuffer[totalHeaderSize+1] == 0x00
+      && fInputBuffer[totalHeaderSize+2] == 0x01) {
     u_int8_t PES_header_data_length = fInputBuffer[totalHeaderSize+8];
     totalHeaderSize += 9 + PES_header_data_length;
     if (totalHeaderSize >= TRANSPORT_PACKET_SIZE) {
@@ -303,7 +310,7 @@ void MPEG2IFrameIndexFromTransportStream::handleInputClosure1() {
     doGetNextFrame();
   } else {
     // Handle closure in the regular way:
-    FramedSource::handleClosure(this);
+    handleClosure();
   }
 }
 
