@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2014 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2016 Live Networks, Inc.  All rights reserved.
 // A parser for a Matroska file.
 // Implementation
 
@@ -100,7 +100,7 @@ void MatroskaFileParser::continueParsing() {
     }
   }
 
-  // We successfully parsed the file's 'Track' headers.  Call our 'done' function now:
+  // We successfully parsed the file.  Call our 'done' function now:
   if (fOnEndFunc != NULL) (*fOnEndFunc)(fOnEndClientData);
 }
 
@@ -433,6 +433,32 @@ Boolean MatroskaFileParser::parseTrack() {
 #endif
 	  if (track != NULL) {
 	    delete[] track->codecID; track->codecID = codecID;
+
+	    // Also set the track's "mimeType" field, if we can deduce it from the "codecID":
+	    if (strncmp(codecID, "A_MPEG", 6) == 0) {
+	      track->mimeType = "audio/MPEG";
+	    } else if (strncmp(codecID, "A_AAC", 5) == 0) {
+	      track->mimeType = "audio/AAC";
+	    } else if (strncmp(codecID, "A_AC3", 5) == 0) {
+	      track->mimeType = "audio/AC3";
+	    } else if (strncmp(codecID, "A_VORBIS", 8) == 0) {
+	      track->mimeType = "audio/VORBIS";
+	    } else if (strcmp(codecID, "A_OPUS") == 0) {
+	      track->mimeType = "audio/OPUS";
+	      track->codecIsOpus = True;
+	    } else if (strcmp(codecID, "V_MPEG4/ISO/AVC") == 0) {
+	      track->mimeType = "video/H264";
+	    } else if (strcmp(codecID, "V_MPEGH/ISO/HEVC") == 0) {
+	      track->mimeType = "video/H265";
+	    } else if (strncmp(codecID, "V_VP8", 5) == 0) {
+	      track->mimeType = "video/VP8";
+	    } else if (strncmp(codecID, "V_VP9", 5) == 0) {
+	      track->mimeType = "video/VP9";
+	    } else if (strncmp(codecID, "V_THEORA", 8) == 0) {
+	      track->mimeType = "video/THEORA";
+	    } else if (strncmp(codecID, "S_TEXT", 6) == 0) {
+	      track->mimeType = "text/T140";
+	    }
 	  } else {
 	    delete[] codecID;
 	  }
@@ -458,7 +484,7 @@ Boolean MatroskaFileParser::parseTrack() {
 	    if (track->codecID != NULL) {
 	      if (strcmp(track->codecID, "V_MPEG4/ISO/AVC") == 0) { // H.264
 		// Byte 4 of the 'codec private' data contains 'lengthSizeMinusOne':
-		if (codecPrivateSize >= 5) track->subframeSizeSize = (codecPrivate[4])&0x3 + 1;
+		if (codecPrivateSize >= 5) track->subframeSizeSize = (codecPrivate[4]&0x3) + 1;
 	      } else if (strcmp(track->codecID, "V_MPEGH/ISO/HEVC") == 0) { // H.265
 		// H.265 'codec private' data is *supposed* to use the format that's described in
 		// http://lists.matroska.org/pipermail/matroska-devel/2013-September/004567.html
@@ -472,13 +498,13 @@ Boolean MatroskaFileParser::parseTrack() {
 		  track->codecPrivateUsesH264FormatForH265 = True;
 		  
 		  // Byte 4 of the 'codec private' data contains 'lengthSizeMinusOne':
-		  if (codecPrivateSize >= 5) track->subframeSizeSize = (codecPrivate[4])&0x3 + 1;
+		  if (codecPrivateSize >= 5) track->subframeSizeSize = (codecPrivate[4]&0x3) + 1;
 		} else {
 		  // This looks like the 'correct' format:
 		  track->codecPrivateUsesH264FormatForH265 = False;
 
 		  // Byte 21 of the 'codec private' data contains 'lengthSizeMinusOne':
-		  track->subframeSizeSize = (codecPrivate[21])&0x3 + 1;
+		  track->subframeSizeSize = (codecPrivate[21]&0x3) + 1;
 		}
 	      }
 	    }
@@ -990,22 +1016,40 @@ Boolean MatroskaFileParser::deliverFrameWithinBlock() {
       return False;
     }
 
-    unsigned frameSize = fFrameSizesWithinBlock[fNextFrameNumberToDeliver];
-    if (track->haveSubframes()) {
-      // The next "track->subframeSizeSize" bytes contain the length of a 'subframe':
-      if (fCurOffsetWithinFrame + track->subframeSizeSize > frameSize) break; // sanity check
-      unsigned subframeSize = 0;
-      for (unsigned i = 0; i < track->subframeSizeSize; ++i) {
-	u_int8_t c;
-	getCommonFrameBytes(track, &c, 1, 0);
-	if (fCurFrameNumBytesToGet > 0) { // it'll be 1
-	  c = get1Byte();
-	  ++fCurOffsetWithinFrame;
+    unsigned frameSize;
+    u_int8_t const* specialFrameSource = NULL;
+    u_int8_t const opusCommentHeader[16]
+      = {'O','p','u','s','T','a','g','s', 0, 0, 0, 0, 0, 0, 0, 0};
+    if (track->codecIsOpus && demuxedTrack->fOpusTrackNumber < 2) {
+      // Special case for Opus audio.  The first frame (the 'configuration' header) comes from
+      // the 'private data'.  The second frame (the 'comment' header) comes is synthesized by
+      // us here:
+      if (demuxedTrack->fOpusTrackNumber == 0) {
+	specialFrameSource = track->codecPrivate;
+	frameSize = track->codecPrivateSize;
+      } else { // demuxedTrack->fOpusTrackNumber == 1
+	specialFrameSource = opusCommentHeader;
+	frameSize = sizeof opusCommentHeader;
+      }
+      ++demuxedTrack->fOpusTrackNumber;
+    } else {
+      frameSize = fFrameSizesWithinBlock[fNextFrameNumberToDeliver];
+      if (track->haveSubframes()) {
+	// The next "track->subframeSizeSize" bytes contain the length of a 'subframe':
+	if (fCurOffsetWithinFrame + track->subframeSizeSize > frameSize) break; // sanity check
+	unsigned subframeSize = 0;
+	for (unsigned i = 0; i < track->subframeSizeSize; ++i) {
+	  u_int8_t c;
+	  getCommonFrameBytes(track, &c, 1, 0);
+	  if (fCurFrameNumBytesToGet > 0) { // it'll be 1
+	    c = get1Byte();
+	    ++fCurOffsetWithinFrame;
+	  }
+	  subframeSize = subframeSize*256 + c;
 	}
-	subframeSize = subframeSize*256 + c;
+	if (subframeSize == 0 || fCurOffsetWithinFrame + subframeSize > frameSize) break; // sanity check
+	frameSize = subframeSize;
       }
-      if (subframeSize == 0 || fCurOffsetWithinFrame + subframeSize > frameSize) break; // sanity check
-      frameSize = subframeSize;
     }
 
     // Compute the presentation time of this frame (from the cluster timecode, the block timecode, and the default duration):
@@ -1023,12 +1067,17 @@ Boolean MatroskaFileParser::deliverFrameWithinBlock() {
     struct timeval presentationTime;
     presentationTime.tv_sec = (unsigned)pt;
     presentationTime.tv_usec = (unsigned)((pt - presentationTime.tv_sec)*1000000);
-    unsigned durationInMicroseconds = track->defaultDuration/1000;
-    if (track->haveSubframes()) {
-      // If this is a 'subframe', use a duration of 0 instead (unless it's the last 'subframe'):
-      if (fCurOffsetWithinFrame + frameSize + track->subframeSizeSize < fFrameSizesWithinBlock[fNextFrameNumberToDeliver]) {
-	// There's room for at least one more subframe after this, so give this subframe a duration of 0
-	durationInMicroseconds = 0;
+    unsigned durationInMicroseconds;
+    if (specialFrameSource != NULL) {
+      durationInMicroseconds = 0;
+    } else { // normal case
+      durationInMicroseconds = track->defaultDuration/1000;
+      if (track->haveSubframes()) {
+	// If this is a 'subframe', use a duration of 0 instead (unless it's the last 'subframe'):
+	if (fCurOffsetWithinFrame + frameSize + track->subframeSizeSize < fFrameSizesWithinBlock[fNextFrameNumberToDeliver]) {
+	  // There's room for at least one more subframe after this, so give this subframe a duration of 0
+	  durationInMicroseconds = 0;
+	}
       }
     }
 
@@ -1069,8 +1118,19 @@ Boolean MatroskaFileParser::deliverFrameWithinBlock() {
     getCommonFrameBytes(track, demuxedTrack->to(), demuxedTrack->frameSize(), demuxedTrack->numTruncatedBytes());
 
     // Next, deliver (and/or skip) bytes from the input file:
-    fCurrentParseState = DELIVERING_FRAME_BYTES;
-    setParseState();
+    if (specialFrameSource != NULL) {
+      memmove(demuxedTrack->to(), specialFrameSource, demuxedTrack->frameSize());
+#ifdef DEBUG
+      fprintf(stderr, "\tdelivered special frame: %d bytes", demuxedTrack->frameSize());
+      if (demuxedTrack->numTruncatedBytes() > 0) fprintf(stderr, " (%d bytes truncated)", demuxedTrack->numTruncatedBytes());
+      fprintf(stderr, " @%u.%06u (%.06f from start); duration %u us\n", demuxedTrack->presentationTime().tv_sec, demuxedTrack->presentationTime().tv_usec, demuxedTrack->presentationTime().tv_sec+demuxedTrack->presentationTime().tv_usec/1000000.0-fPresentationTimeOffset, demuxedTrack->durationInMicroseconds());
+#endif
+      setParseState();
+      FramedSource::afterGetting(demuxedTrack); // completes delivery
+    } else { // normal case
+      fCurrentParseState = DELIVERING_FRAME_BYTES;
+      setParseState();
+    }
     return True;
   } while (0);
 
