@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2014 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2016 Live Networks, Inc.  All rights reserved.
 // A sink that generates a QuickTime file from a composite media session
 // Implementation
 
@@ -318,7 +318,7 @@ QuickTimeFileSink::~QuickTimeFileSink() {
   MediaSubsessionIterator iter(fInputSession);
   MediaSubsession* subsession;
   while ((subsession = iter.next()) != NULL) {
-    subsession->readSource()->stopGettingFrames();
+    if (subsession->readSource() != NULL) subsession->readSource()->stopGettingFrames();
 
     SubsessionIOState* ioState
       = (SubsessionIOState*)(subsession->miscPtr);
@@ -355,6 +355,12 @@ QuickTimeFileSink::createNew(UsageEnvironment& env,
   return newSink;
 }
 
+void QuickTimeFileSink
+::noteRecordedFrame(MediaSubsession& /*inputSubsession*/,
+		    unsigned /*packetDataSize*/, struct timeval const& /*presentationTime*/) {
+  // Default implementation: Do nothing
+}
+
 Boolean QuickTimeFileSink::startPlaying(afterPlayingFunc* afterFunc,
 					void* afterClientData) {
   // Make sure we're not already being played:
@@ -705,18 +711,22 @@ void SubsessionIOState::afterGettingFrame(unsigned packetDataSize,
 					  struct timeval presentationTime) {
   // Begin by checking whether there was a gap in the RTP stream.
   // If so, try to compensate for this (if desired):
-  unsigned short rtpSeqNum
-    = fOurSubsession.rtpSource()->curPacketRTPSeqNum();
-  if (fOurSink.fPacketLossCompensate && fPrevBuffer->bytesInUse() > 0) {
-    short seqNumGap = rtpSeqNum - fLastPacketRTPSeqNum;
-    for (short i = 1; i < seqNumGap; ++i) {
-      // Insert a copy of the previous frame, to compensate for the loss:
-      useFrame(*fPrevBuffer);
+  if (fOurSubsession.rtpSource() != NULL) { // we have a RTP stream
+    unsigned short rtpSeqNum
+      = fOurSubsession.rtpSource()->curPacketRTPSeqNum();
+    if (fOurSink.fPacketLossCompensate && fPrevBuffer->bytesInUse() > 0) {
+      short seqNumGap = rtpSeqNum - fLastPacketRTPSeqNum;
+      for (short i = 1; i < seqNumGap; ++i) {
+	// Insert a copy of the previous frame, to compensate for the loss:
+	useFrame(*fPrevBuffer);
+      }
     }
+    fLastPacketRTPSeqNum = rtpSeqNum;
   }
-  fLastPacketRTPSeqNum = rtpSeqNum;
 
   // Now, continue working with the frame that we just got
+  fOurSink.noteRecordedFrame(fOurSubsession, packetDataSize, presentationTime);
+
   if (fBuffer->bytesInUse() == 0) {
     fBuffer->setPresentationTime(presentationTime);
   }
@@ -724,7 +734,8 @@ void SubsessionIOState::afterGettingFrame(unsigned packetDataSize,
 
   // If our RTP source is a "QuickTimeGenericRTPSource", then
   // use its 'qtState' to set some parameters that we need:
-  if (fQTMediaDataAtomCreator == &QuickTimeFileSink::addAtom_genericMedia){
+  if (fOurSubsession.rtpSource() != NULL // we have a RTP stream
+      && fQTMediaDataAtomCreator == &QuickTimeFileSink::addAtom_genericMedia) {
     QuickTimeGenericRTPSource* rtpSource
       = (QuickTimeGenericRTPSource*)fOurSubsession.rtpSource();
     QuickTimeGenericRTPSource::QTState& qtState = rtpSource->qtState;
@@ -843,13 +854,12 @@ void SubsessionIOState::useFrame(SubsessionBuffer& buffer) {
   // Write the data into the file:
   fwrite(frameSource, 1, frameSize, fOurSink.fOutFid);
 
-  // If we have a hint track, then write to it also:
-  if (hasHintTrack()) {
+  // If we have a hint track, then write to it also (only if we have a RTP stream):
+  if (hasHintTrack() && fOurSubsession.rtpSource() != NULL) {
     // Because presentation times are used for RTP packet timestamps,
     // we don't starting writing to the hint track until we've been synced:
     if (!fHaveBeenSynced) {
-      fHaveBeenSynced
-	= fOurSubsession.rtpSource()->hasBeenSynchronizedUsingRTCP();
+      fHaveBeenSynced = fOurSubsession.rtpSource()->hasBeenSynchronizedUsingRTCP();
     }
     if (fHaveBeenSynced) {
       fHintTrackForUs->useFrameForHinting(frameSize, presentationTime,
@@ -876,7 +886,7 @@ void SubsessionIOState::useFrameForHinting(unsigned frameSize,
   // If there has been a previous frame, then output a 'hint sample' for it.
   // (We use the current frame's presentation time to compute the previous
   // hint sample's duration.)
-  RTPSource* const rs = fOurSubsession.rtpSource(); // abbrev
+  RTPSource* const rs = fOurSubsession.rtpSource(); // abbrev (ASSERT: != NULL)
   struct timeval const& ppt = fPrevFrameState.presentationTime; //abbrev
   if (ppt.tv_sec != 0 || ppt.tv_usec != 0) {
     double duration = (presentationTime.tv_sec - ppt.tv_sec)
@@ -1036,8 +1046,8 @@ void SubsessionIOState::useFrameForHinting(unsigned frameSize,
     }
   } else if (hackm4a_generic) {
     // Synthesize a special header, so that this frame can be in its own RTP packet.
-    unsigned const sizeLength = fOurSubsession.fmtp_sizelength();
-    unsigned const indexLength = fOurSubsession.fmtp_indexlength();
+    unsigned const sizeLength = fOurSubsession.attrVal_unsigned("sizelength");
+    unsigned const indexLength = fOurSubsession.attrVal_unsigned("indexlength");
     if (sizeLength + indexLength != 16) {
       envir() << "Warning: unexpected 'sizeLength' " << sizeLength
 	      << " and 'indexLength' " << indexLength
@@ -1094,7 +1104,7 @@ void SubsessionIOState::onSourceClosure() {
 
 Boolean SubsessionIOState::syncOK(struct timeval presentationTime) {
   QuickTimeFileSink& s = fOurSink; // abbreviation
-  if (!s.fSyncStreams) return True; // we don't care
+  if (!s.fSyncStreams || fOurSubsession.rtpSource() == NULL) return True; // we don't care
 
   if (s.fNumSyncedSubsessions < s.fNumSubsessions) {
     // Not all subsessions have yet been synced.  Check whether ours was
@@ -1667,10 +1677,13 @@ unsigned QuickTimeFileSink::addAtom_genericMedia() {
   // Use its "sdAtom" state for our contents:
   QuickTimeGenericRTPSource* rtpSource = (QuickTimeGenericRTPSource*)
     fCurrentIOState->fOurSubsession.rtpSource();
-  QuickTimeGenericRTPSource::QTState& qtState = rtpSource->qtState;
-  char const* from = qtState.sdAtom;
-  unsigned size = qtState.sdAtomSize;
-  for (unsigned i = 0; i < size; ++i) addByte(from[i]);
+  unsigned size = 0;
+  if (rtpSource != NULL) {
+    QuickTimeGenericRTPSource::QTState& qtState = rtpSource->qtState;
+    char const* from = qtState.sdAtom;
+    size = qtState.sdAtomSize;
+    for (unsigned i = 0; i < size; ++i) addByte(from[i]);
+  }
 addAtomEnd;
 
 unsigned QuickTimeFileSink::addAtom_soundMediaGeneral() {
@@ -2010,13 +2023,20 @@ addAtom(stss); // Sync-Sample
   unsigned numEntries = 0, numSamplesSoFar = 0;
   if (fCurrentIOState->fHeadSyncFrame != NULL) {
     SyncFrame* currentSyncFrame = fCurrentIOState->fHeadSyncFrame;
-    while(currentSyncFrame != NULL) {
+
+    // First, count the number of frames (to use as a sanity check; see below):
+    unsigned totNumFrames = 0;
+    for (ChunkDescriptor* chunk = fCurrentIOState->fHeadChunk; chunk != NULL; chunk = chunk->fNextChunk) totNumFrames += chunk->fNumFrames;
+
+    while (currentSyncFrame != NULL) {
+      if (currentSyncFrame->sfFrameNum >= totNumFrames) break; // sanity check
+      
       ++numEntries;
       size += addWord(currentSyncFrame->sfFrameNum);
       currentSyncFrame = currentSyncFrame->nextSyncFrame;
     }
   } else {
-    // Then, run through the chunk descriptors, counting up the total nuber of samples:
+    // First, run through the chunk descriptors, counting up the total number of samples:
     unsigned const samplesPerFrame = fCurrentIOState->fQTSamplesPerFrame;
     ChunkDescriptor* chunk = fCurrentIOState->fHeadChunk;
     while (chunk != NULL) {
@@ -2299,15 +2319,17 @@ addAtomEnd;
 addAtom(payt);
   MediaSubsession& ourSubsession = fCurrentIOState->fOurSubsession;
   RTPSource* rtpSource = ourSubsession.rtpSource();
-  size += addWord(rtpSource->rtpPayloadFormat());
+  if (rtpSource != NULL) {
+    size += addWord(rtpSource->rtpPayloadFormat());
 
-  // Also, add a 'rtpmap' string: <mime-subtype>/<rtp-frequency>
-  unsigned rtpmapStringLength = strlen(ourSubsession.codecName()) + 20;
-  char* rtpmapString = new char[rtpmapStringLength];
-  sprintf(rtpmapString, "%s/%d",
-	  ourSubsession.codecName(), rtpSource->timestampFrequency());
-  size += addArbitraryString(rtpmapString);
-  delete[] rtpmapString;
+    // Also, add a 'rtpmap' string: <mime-subtype>/<rtp-frequency>
+    unsigned rtpmapStringLength = strlen(ourSubsession.codecName()) + 20;
+    char* rtpmapString = new char[rtpmapStringLength];
+    sprintf(rtpmapString, "%s/%d",
+	    ourSubsession.codecName(), rtpSource->timestampFrequency());
+    size += addArbitraryString(rtpmapString);
+    delete[] rtpmapString;
+  }
 addAtomEnd;
 
 // A dummy atom (with name "????"):
