@@ -14,94 +14,101 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2014 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2016 Live Networks, Inc.  All rights reserved.
 // RTP sink for Vorbis audio
 // Implementation
 
 #include "VorbisAudioRTPSink.hh"
 #include "Base64.hh"
+#include "VorbisAudioRTPSource.hh" // for parseVorbisOrTheoraConfigStr()
 
-VorbisAudioRTPSink::VorbisAudioRTPSink(UsageEnvironment& env, Groupsock* RTPgs,
-				       u_int8_t rtpPayloadFormat,
-				       u_int32_t rtpTimestampFrequency,
-				       unsigned numChannels,
-				       u_int8_t* identificationHeader, unsigned identificationHeaderSize,
-				       u_int8_t* commentHeader, unsigned commentHeaderSize,
-				       u_int8_t* setupHeader, unsigned setupHeaderSize,
-				       u_int32_t identField)
+VorbisAudioRTPSink* VorbisAudioRTPSink
+::createNew(UsageEnvironment& env, Groupsock* RTPgs,
+	    u_int8_t rtpPayloadFormat, u_int32_t rtpTimestampFrequency, unsigned numChannels,
+	    u_int8_t* identificationHeader, unsigned identificationHeaderSize,
+	    u_int8_t* commentHeader, unsigned commentHeaderSize,
+	    u_int8_t* setupHeader, unsigned setupHeaderSize,
+	    u_int32_t identField) {
+  return new VorbisAudioRTPSink(env, RTPgs,
+				rtpPayloadFormat, rtpTimestampFrequency, numChannels,
+				identificationHeader, identificationHeaderSize,
+				commentHeader, commentHeaderSize,
+				setupHeader, setupHeaderSize,
+				identField);
+}
+
+VorbisAudioRTPSink* VorbisAudioRTPSink
+::createNew(UsageEnvironment& env, Groupsock* RTPgs,u_int8_t rtpPayloadFormat,
+	    u_int32_t rtpTimestampFrequency, unsigned numChannels,
+	    char const* configStr) {
+  // Begin by decoding and unpacking the configuration string:
+  u_int8_t* identificationHeader; unsigned identificationHeaderSize;
+  u_int8_t* commentHeader; unsigned commentHeaderSize;
+  u_int8_t* setupHeader; unsigned setupHeaderSize;
+  u_int32_t identField;
+
+  parseVorbisOrTheoraConfigStr(configStr,
+			       identificationHeader, identificationHeaderSize,
+			       commentHeader, commentHeaderSize,
+			       setupHeader, setupHeaderSize,
+			       identField);
+
+  VorbisAudioRTPSink* resultSink
+    = new VorbisAudioRTPSink(env, RTPgs, rtpPayloadFormat, rtpTimestampFrequency, numChannels,
+			     identificationHeader, identificationHeaderSize,
+			     commentHeader, commentHeaderSize,
+			     setupHeader, setupHeaderSize,
+			     identField);
+  delete[] identificationHeader; delete[] commentHeader; delete[] setupHeader;
+
+  return resultSink;
+}
+
+VorbisAudioRTPSink
+::VorbisAudioRTPSink(UsageEnvironment& env, Groupsock* RTPgs, u_int8_t rtpPayloadFormat,
+		     u_int32_t rtpTimestampFrequency, unsigned numChannels,
+		     u_int8_t* identificationHeader, unsigned identificationHeaderSize,
+		     u_int8_t* commentHeader, unsigned commentHeaderSize,
+		     u_int8_t* setupHeader, unsigned setupHeaderSize,
+		     u_int32_t identField)
   : AudioRTPSink(env, RTPgs, rtpPayloadFormat, rtpTimestampFrequency, "VORBIS", numChannels),
     fIdent(identField), fFmtpSDPLine(NULL) {
-  // Create packed configuration headers, and encode this data into a "a=fmtp:" SDP line that we'll use to describe it:
+  if (identificationHeaderSize >= 28) {
+    // Get the 'bitrate' values from this header, and use them to set our estimated bitrate:
+    u_int32_t val;
+    u_int8_t* p;
+    
+    p = &identificationHeader[16];
+    val = ((p[3]*256 + p[2])*256 + p[1])*256 + p[0]; // i.e., little-endian
+    int bitrate_maximum = (int)val;
+    if (bitrate_maximum < 0) bitrate_maximum = 0;
+    
+    p = &identificationHeader[20];
+    val = ((p[3]*256 + p[2])*256 + p[1])*256 + p[0]; // i.e., little-endian
+    int bitrate_nominal = (int)val;
+    if (bitrate_nominal < 0) bitrate_nominal = 0;
+    
+    p = &identificationHeader[24];
+    val = ((p[3]*256 + p[2])*256 + p[1])*256 + p[0]; // i.e., little-endian
+    int bitrate_minimum = (int)val;
+    if (bitrate_minimum < 0) bitrate_minimum = 0;
+    
+    int bitrate
+      = bitrate_nominal > 0 ? bitrate_nominal
+      : bitrate_maximum > 0 ? bitrate_maximum
+      : bitrate_minimum > 0 ? bitrate_minimum : 0;
+    if (bitrate > 0) estimatedBitrate() = ((unsigned)bitrate)/1000;
+  }
   
-  // First, count how many headers (<=3) are included, and how many bytes will be used to encode these headers' sizes:
-  unsigned numHeaders = 0;
-  unsigned sizeSize[2]; // The number of bytes used to encode the lengths of the first two headers (but not the length of the 3rd)
-  sizeSize[0] = sizeSize[1] = 0;
-  if (identificationHeaderSize > 0) {
-    sizeSize[numHeaders++] = identificationHeaderSize < 128 ? 1 : identificationHeaderSize < 16384 ? 2 : 3;
-  }
-  if (commentHeaderSize > 0) {
-    sizeSize[numHeaders++] = commentHeaderSize < 128 ? 1 : commentHeaderSize < 16384 ? 2 : 3;
-  }
-  if (setupHeaderSize > 0) {
-    ++numHeaders;
-  } else {
-    sizeSize[1] = 0; // We have at most two headers, so the second one's length isn't encoded
-  }
-  if (numHeaders == 0) return; // With no headers, we can't set up a configuration
-  if (numHeaders == 1) sizeSize[0] = 0; // With only one header, its length isn't encoded
-
-  // Then figure out the size of the packed configuration headers, and allocate space for this:
-  unsigned length = identificationHeaderSize + commentHeaderSize + setupHeaderSize; // The "length" field in the packed headers
-  if (length > (unsigned)0xFFFF) return; // too big for a 16-bit field; we can't handle this
-  unsigned packedHeadersSize
-    = 4 // "Number of packed headers" field
-    + 3 // "ident" field
-    + 2 // "length" field
-    + 1 // "n. of headers" field
-    + sizeSize[0] + sizeSize[1] // "length1" and "length2" (if present) fields
-    + length;
-  u_int8_t* packedHeaders = new u_int8_t[packedHeadersSize];
-  if (packedHeaders == NULL) return;
-
-  // Fill in the 'packed headers':
-  u_int8_t* p = packedHeaders;
-  *p++ = 0; *p++ = 0; *p++ = 0; *p++ = 1; // "Number of packed headers": 1
-  *p++ = fIdent>>16; *p++ = fIdent>>8; *p++ = fIdent; // "Ident" (24 bits)
-  *p++ = length>>8; *p++ = length; // "length" (16 bits)
-  *p++ = numHeaders-1; // "n. of headers"
-  if (numHeaders > 1) {
-    // Fill in the "length1" header:
-    unsigned length1 = identificationHeaderSize > 0 ? identificationHeaderSize : commentHeaderSize;
-    if (length1 >= 16384) {
-      *p++ = 0x80; // flag, but no more, because we know length1 <= 32767
-    }
-    if (length1 >= 128) {
-      *p++ = 0x80|((length1&0x3F80)>>7); // flag + the second 7 bits
-    }
-    *p++ = length1&0x7F; // the low 7 bits
-
-    if (numHeaders > 2) { // numHeaders == 3
-      // Fill in the "length2" header (for the 'Comment' header):
-      unsigned length2 = commentHeaderSize;
-      if (length2 >= 16384) {
-	*p++ = 0x80; // flag, but no more, because we know length2 <= 32767
-      }
-      if (length2 >= 128) {
-	*p++ = 0x80|((length2&0x3F80)>>7); // flag + the second 7 bits
-      }
-      *p++ = length2&0x7F; // the low 7 bits
-    }
-  }
-  // Copy each header:
-  if (identificationHeader != NULL) memmove(p, identificationHeader, identificationHeaderSize); p += identificationHeaderSize;
-  if (commentHeader != NULL) memmove(p, commentHeader, commentHeaderSize); p += commentHeaderSize;
-  if (setupHeader != NULL) memmove(p, setupHeader, setupHeaderSize);
-  
-  // Having set up the 'packed configuration headers', Base-64-encode this, and put it in our "a=fmtp:" SDP line:
-  char* base64PackedHeaders = base64Encode((char const*)packedHeaders, packedHeadersSize);
-  delete[] packedHeaders;
+  // Generate a 'config' string from the supplied configuration headers:
+  char* base64PackedHeaders
+    = generateVorbisOrTheoraConfigStr(identificationHeader, identificationHeaderSize,
+				      commentHeader, commentHeaderSize,
+				      setupHeader, setupHeaderSize,
+				      identField);
+  if (base64PackedHeaders == NULL) return;
   
+  // Then use this 'config' string to construct our "a=fmtp:" SDP line:
   unsigned fmtpSDPLineMaxSize = 50 + strlen(base64PackedHeaders); // 50 => more than enough space
   fFmtpSDPLine = new char[fmtpSDPLineMaxSize];
   sprintf(fFmtpSDPLine, "a=fmtp:%d configuration=%s\r\n", rtpPayloadType(), base64PackedHeaders);
@@ -112,106 +119,6 @@ VorbisAudioRTPSink::~VorbisAudioRTPSink() {
   delete[] fFmtpSDPLine;
 }
 
-VorbisAudioRTPSink*
-VorbisAudioRTPSink::createNew(UsageEnvironment& env, Groupsock* RTPgs,
-			      u_int8_t rtpPayloadFormat, u_int32_t rtpTimestampFrequency, unsigned numChannels,
-			      u_int8_t* identificationHeader, unsigned identificationHeaderSize,
-			      u_int8_t* commentHeader, unsigned commentHeaderSize,
-			      u_int8_t* setupHeader, unsigned setupHeaderSize) {
-  return new VorbisAudioRTPSink(env, RTPgs,
-				rtpPayloadFormat, rtpTimestampFrequency, numChannels,
-				identificationHeader, identificationHeaderSize,
-				commentHeader, commentHeaderSize,
-				setupHeader, setupHeaderSize);
-}
-
-#define ADVANCE(n) do { p += (n); rem -= (n); } while (0)
-#define GET_ENCODED_VAL(n) do { u_int8_t byte; n = 0; do { if (rem == 0) break; byte = *p; n = (n*128) + (byte&0x7F); ADVANCE(1); } while (byte&0x80); } while (0); if (rem == 0) break
-
-VorbisAudioRTPSink*
-VorbisAudioRTPSink::createNew(UsageEnvironment& env, Groupsock* RTPgs,
-			      u_int8_t rtpPayloadFormat, u_int32_t rtpTimestampFrequency, unsigned numChannels,
-			      char const* configStr) {
-  u_int8_t* identificationHeader = NULL; unsigned identificationHeaderSize = 0;
-  u_int8_t* commentHeader = NULL; unsigned commentHeaderSize = 0;
-  u_int8_t* setupHeader = NULL; unsigned setupHeaderSize = 0;
-  VorbisAudioRTPSink* resultSink = NULL;
-
-  // Begin by Base64-decoding the configuration string:
-  unsigned configDataSize;
-  u_int8_t* configData = base64Decode(configStr, configDataSize);
-  u_int8_t* p = configData;
-  unsigned rem = configDataSize;
-
-  do {
-    if (rem < 4) break;
-    u_int32_t numPackedHeaders = (p[0]<<24)|(p[1]<<16)|(p[2]<<8)|p[3]; ADVANCE(4);
-
-    if (numPackedHeaders == 0) break;
-    // Use the first 'packed header' only.
-
-    if (rem < 3) break;
-    u_int32_t ident = (p[0]<<16)|(p[1]<<8)|p[2]; ADVANCE(3);
-
-    if (rem < 2) break;
-    u_int16_t length = (p[0]<<8)|p[1]; ADVANCE(2);
-
-    unsigned numHeaders;
-    GET_ENCODED_VAL(numHeaders);
-
-    Boolean success = False;
-    for (unsigned i = 0; i < numHeaders+1 && i < 3; ++i) {
-      success = False;
-      unsigned headerSize;
-      if (i < numHeaders) {
-	// The header size is encoded:
-	GET_ENCODED_VAL(headerSize);
-	if (headerSize > length) break;
-	length -= headerSize;
-      } else {
-	// The last header is implicit:
-	headerSize = length;
-      }
-
-      // Allocate space for the header bytes; we'll fill it in later
-      if (i == 0) {
-	identificationHeaderSize = headerSize;
-	identificationHeader = new u_int8_t[identificationHeaderSize];
-      } else if (i == 1) {
-	commentHeaderSize = headerSize;
-	commentHeader = new u_int8_t[commentHeaderSize];
-      } else { // i == 2
-	setupHeaderSize = headerSize;
-	setupHeader = new u_int8_t[setupHeaderSize];
-      }
-
-      success = True;
-    }
-    if (!success) break;
-
-    // Copy the remaining config bytes into the appropriate 'header' buffers:
-    if (identificationHeader != NULL) {
-      memmove(identificationHeader, p, identificationHeaderSize); ADVANCE(identificationHeaderSize);
-      if (commentHeader != NULL) {
-	memmove(commentHeader, p, commentHeaderSize); ADVANCE(commentHeaderSize);
-	if (setupHeader != NULL) {
-	  memmove(setupHeader, p, setupHeaderSize); ADVANCE(setupHeaderSize);
-	}
-      }
-    }
-
-    resultSink = new VorbisAudioRTPSink(env, RTPgs, rtpPayloadFormat, rtpTimestampFrequency, numChannels,
-					identificationHeader, identificationHeaderSize,
-					commentHeader, commentHeaderSize,
-					setupHeader, setupHeaderSize,
-					ident);
-  } while (0);
-
-  delete[] configData;
-  delete[] identificationHeader; delete[] commentHeader; delete[] setupHeader;
-  return resultSink;
-}
-
 char const* VorbisAudioRTPSink::auxSDPLine() {
   return fFmtpSDPLine;
 }
@@ -225,7 +132,7 @@ void VorbisAudioRTPSink
   // Set the 4-byte "payload header", as defined in RFC 5215, section 2.2:
   u_int8_t header[4];
 
-  // The three bytes of the header are our "Ident":
+  // The first three bytes of the header are our "Ident":
   header[0] = fIdent>>16; header[1] = fIdent>>8; header[2] = fIdent;
 
   // The final byte contains the "F", "VDT", and "numPkts" fields:
@@ -276,3 +183,84 @@ unsigned VorbisAudioRTPSink::specialHeaderSize() const {
 unsigned VorbisAudioRTPSink::frameSpecificHeaderSize() const {
   return 2;
 }
+
+
+////////// generateVorbisOrTheoraConfigStr() implementation //////////
+
+char* generateVorbisOrTheoraConfigStr(u_int8_t* identificationHeader, unsigned identificationHeaderSize,
+                                      u_int8_t* commentHeader, unsigned commentHeaderSize,
+                                      u_int8_t* setupHeader, unsigned setupHeaderSize,
+                                      u_int32_t identField) {
+  // First, count how many headers (<=3) are included, and how many bytes will be used
+  // to encode these headers' sizes:
+  unsigned numHeaders = 0;
+  unsigned sizeSize[2]; // The number of bytes used to encode the lengths of the first two headers (but not the length of the 3rd)
+  sizeSize[0] = sizeSize[1] = 0;
+  if (identificationHeaderSize > 0) {
+    sizeSize[numHeaders++] = identificationHeaderSize < 128 ? 1 : identificationHeaderSize < 16384 ? 2 : 3;
+  }
+  if (commentHeaderSize > 0) {
+    sizeSize[numHeaders++] = commentHeaderSize < 128 ? 1 : commentHeaderSize < 16384 ? 2 : 3;
+  }
+  if (setupHeaderSize > 0) {
+    ++numHeaders;
+  } else {
+    sizeSize[1] = 0; // We have at most two headers, so the second one's length isn't encoded
+  }
+  if (numHeaders == 0) return NULL; // With no headers, we can't set up a configuration
+  if (numHeaders == 1) sizeSize[0] = 0; // With only one header, its length isn't encoded
+
+  // Then figure out the size of the packed configuration headers, and allocate space for this:
+  unsigned length = identificationHeaderSize + commentHeaderSize + setupHeaderSize;
+      // The "length" field in the packed headers
+  if (length > (unsigned)0xFFFF) return NULL; // too big for a 16-bit field; we can't handle this
+  unsigned packedHeadersSize
+    = 4 // "Number of packed headers" field
+    + 3 // "ident" field
+    + 2 // "length" field
+    + 1 // "n. of headers" field
+    + sizeSize[0] + sizeSize[1] // "length1" and "length2" (if present) fields
+    + length;
+  u_int8_t* packedHeaders = new u_int8_t[packedHeadersSize];
+  if (packedHeaders == NULL) return NULL;
+
+  // Fill in the 'packed headers':
+  u_int8_t* p = packedHeaders;
+  *p++ = 0; *p++ = 0; *p++ = 0; *p++ = 1; // "Number of packed headers": 1
+  *p++ = identField>>16; *p++ = identField>>8; *p++ = identField; // "Ident" (24 bits)
+  *p++ = length>>8; *p++ = length; // "length" (16 bits)
+  *p++ = numHeaders-1; // "n. of headers"
+  if (numHeaders > 1) {
+    // Fill in the "length1" header:
+    unsigned length1 = identificationHeaderSize > 0 ? identificationHeaderSize : commentHeaderSize;
+    if (length1 >= 16384) {
+      *p++ = 0x80; // flag, but no more, because we know length1 <= 32767
+    }
+    if (length1 >= 128) {
+      *p++ = 0x80|((length1&0x3F80)>>7); // flag + the second 7 bits
+    }
+    *p++ = length1&0x7F; // the low 7 bits
+
+    if (numHeaders > 2) { // numHeaders == 3
+      // Fill in the "length2" header (for the 'Comment' header):
+      unsigned length2 = commentHeaderSize;
+      if (length2 >= 16384) {
+	*p++ = 0x80; // flag, but no more, because we know length2 <= 32767
+      }
+      if (length2 >= 128) {
+	*p++ = 0x80|((length2&0x3F80)>>7); // flag + the second 7 bits
+      }
+      *p++ = length2&0x7F; // the low 7 bits
+    }
+  }
+  // Copy each header:
+  if (identificationHeader != NULL) memmove(p, identificationHeader, identificationHeaderSize); p += identificationHeaderSize;
+  if (commentHeader != NULL) memmove(p, commentHeader, commentHeaderSize); p += commentHeaderSize;
+  if (setupHeader != NULL) memmove(p, setupHeader, setupHeaderSize);
+  
+  // Having set up the 'packed configuration headers', Base-64-encode this, for our result:
+  char* base64PackedHeaders = base64Encode((char const*)packedHeaders, packedHeadersSize);
+  delete[] packedHeaders;
+  
+  return base64PackedHeaders;
+}
