@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2014 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2016 Live Networks, Inc.  All rights reserved.
 // A WAV audio file source
 // Implementation
 
@@ -65,12 +65,14 @@ void WAVAudioFileSource::setScaleFactor(int scale) {
   }
 }
 
-void WAVAudioFileSource::seekToPCMByte(unsigned byteNumber, unsigned numBytesToStream) {
+void WAVAudioFileSource::seekToPCMByte(unsigned byteNumber) {
   byteNumber += fWAVHeaderSize;
   if (byteNumber > fFileSize) byteNumber = fFileSize;
 
   SeekFile64(fFid, byteNumber, SEEK_SET);
+}
 
+void WAVAudioFileSource::limitNumBytesToStream(unsigned numBytesToStream) {
   fNumBytesToStream = numBytesToStream;
   fLimitNumBytesToStream = fNumBytesToStream > 0;
 }
@@ -222,7 +224,7 @@ WAVAudioFileSource::~WAVAudioFileSource() {
 
 void WAVAudioFileSource::doGetNextFrame() {
   if (feof(fFid) || ferror(fFid) || (fLimitNumBytesToStream && fNumBytesToStream == 0)) {
-    handleClosure(this);
+    handleClosure();
     return;
   }
 
@@ -240,6 +242,7 @@ void WAVAudioFileSource::doGetNextFrame() {
 }
 
 void WAVAudioFileSource::doStopGettingFrames() {
+  envir().taskScheduler().unscheduleDelayedTask(nextTask());
 #ifndef READ_FROM_FILES_SYNCHRONOUSLY
   envir().taskScheduler().turnOffBackgroundReadHandling(fileno(fFid));
   fHaveStartedReading = False;
@@ -280,7 +283,7 @@ void WAVAudioFileSource::doReadFromFile() {
     }
 #endif
     if (numBytesRead == 0) {
-     handleClosure(this);
+     handleClosure();
       return;
     }
     fFrameSize += numBytesRead;
